﻿using System.ComponentModel;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.QuadraticAssignment {
  [Item("QAP Assignment", "Represents a solution to the QAP.")]
  [StorableClass]
  public sealed class QAPAssignment : Item, INotifyPropertyChanged {
    [Storable]
    private DoubleMatrix coordinates;
    public DoubleMatrix Coordinates {
      get { return coordinates; }
      set {
        bool changed = (coordinates != value);
        coordinates = value;
        if (changed) OnPropertyChanged("Coordinates");
      }
    }

    public DoubleMatrix ViewCoordinates { get; set; }

    [Storable]
    private DoubleMatrix distances;
    public DoubleMatrix Distances {
      get { return distances; }
      set {
        bool changed = (distances != value);
        distances = value;
        if (changed) OnPropertyChanged("Distances");
      }
    }

    [Storable]
    private DoubleMatrix weights;
    public DoubleMatrix Weights {
      get { return weights; }
      set {
        bool changed = (weights != value);
        weights = value;
        if (changed) OnPropertyChanged("Weights");
      }
    }

    [Storable]
    private Permutation assignment;
    public Permutation Assignment {
      get { return assignment; }
      set {
        bool changed = (assignment != value);
        assignment = value;
        if (changed) OnPropertyChanged("Assignment");
      }
    }

    [Storable]
    private DoubleValue quality;
    public DoubleValue Quality {
      get { return quality; }
      set {
        bool changed = (quality != value);
        quality = value;
        if (changed) OnPropertyChanged("Quality");
      }
    }

    [StorableConstructor]
    private QAPAssignment(bool deserializing) : base(deserializing) { }
    private QAPAssignment(QAPAssignment original, Cloner cloner)
      : base(original, cloner) {
      coordinates = cloner.Clone(original.coordinates);
      distances = cloner.Clone(original.distances);
      weights = cloner.Clone(original.weights);
      assignment = cloner.Clone(original.assignment);
      quality = cloner.Clone(original.quality);
    }
    public QAPAssignment(DoubleMatrix weights, Permutation assignment) {
      this.weights = weights;
      this.assignment = assignment;
    }
    public QAPAssignment(DoubleMatrix weights, Permutation assignment, DoubleValue quality)
      : this(weights, assignment) {
      this.quality = quality;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new QAPAssignment(this, cloner);
    }


    public event PropertyChangedEventHandler PropertyChanged;
    private void OnPropertyChanged(string propertyName) {
      var handler = PropertyChanged;
      if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
    }
  }
}
